
$logFile = "C:\Simple\Logs\win10.log"
$regFile = "C:\Simple\pmac.reg"

Function LogWrite
{
	Param ([string]$logstring)
   
	# check if logfile is getting too big
	if ((get-item $logFile).length -gt 500kb) {
		write-host "truncating log file"
		$logContents = Get-Content $logFile -Tail 100
		$logContents | Out-File -FilePath $logFile
	}

	# log to console and file
	$logstring = $(get-date -f "yyyy-MM-dd HH:mm:ss") + "  " + $logstring 
	Add-content $logFile -value $logstring
	write-host $logstring
}

function set-pmac-perms {
	# Check if PMAC registry section has FullControl enabled for BUILTIN\USERS.
	$PMACPermsOK = "0"
	foreach ($item in ((Get-Item "HKLM:\SYSTEM\CurrentControlSet\Services\PMAC").GetAccessControl('Access').Access)) {
		If ($item.IdentityReference -eq "BUILTIN\USERS") {
			if ($item.RegistryRights -eq "FullControl") {
				if ($item.AccessControlType -eq "Allow") {
					$PMACPermsOK = "1"
					LogWrite "PMAC Registry permissions OK"
				}
			}
		}
	}
	If ($PMACPermsOK -eq "0") {
		#LogWrite("PMAC Registry permissions are not correct.")
		LogWrite("Setting permissions to PMAC registry to allow users full control...")
		$acl= get-acl -path "hklm:\SYSTEM\CurrentControlSet\Services\PMAC"
		$inherit = [system.security.accesscontrol.InheritanceFlags]"ContainerInherit, ObjectInherit"
		$propagation = [system.security.accesscontrol.PropagationFlags]"None"
		$rule=new-object system.security.accesscontrol.registryaccessrule "BUILTIN\Users","FullControl",$inherit,$propagation,"Allow"
		$acl.addaccessrule($rule)
		$acl|set-acl
	}
}

LogWrite("Windows build: " + [System.Environment]::OSVersion.Version.Build)

# Check if registry key exists
IF (Test-Path -Path "HKLM:\SYSTEM\CurrentControlSet\Services\PMAC\Device0\") {
	#LogWrite("PMAC Registry entries found")
	set-pmac-perms
	
	#export reg keys
	LogWrite("Backing up PMAC registry")
	reg export "HKey_Local_Machine\SYSTEM\CurrentControlSet\Services\PMAC" $regFile /y
	
} else {
	LogWrite("PMAC Registry entries NOT found, restoring from backup")
	
	#kill PmacServer process just in case
	Get-Process pmacse* | Stop-Process -force
	
	#restore registry entries from backup
	reg import $regFile
	
	#set registry permissions
	set-pmac-perms
}
